@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
  *
  */
 
+#include "config.h"
 #include "idle.h"
 #include "event_pipe.h"
 
@@ -40,6 +41,7 @@ static const char *const idle_names[] = {
 	"output",
 	"options",
 	"sticker",
+	"update",
         NULL
 };
 
