@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "config.h"
 #include "input/mms_input_plugin.h"
 #include "input_plugin.h"
 
@@ -30,13 +31,21 @@
 #define G_LOG_DOMAIN "input_mms"
 
 struct input_mms {
+	struct input_stream base;
+
 	mmsx_t *mms;
 
 	bool eof;
 };
 
-static bool
-input_mms_open(struct input_stream *is, const char *url)
+static inline GQuark
+mms_quark(void)
+{
+	return g_quark_from_static_string("mms");
+}
+
+static struct input_stream *
+input_mms_open(const char *url, GError **error_r)
 {
 	struct input_mms *m;
 
@@ -44,39 +53,42 @@ input_mms_open(struct input_stream *is, const char *url)
 	    !g_str_has_prefix(url, "mmsh://") &&
 	    !g_str_has_prefix(url, "mmst://") &&
 	    !g_str_has_prefix(url, "mmsu://"))
-		return false;
+		return NULL;
 
 	m = g_new(struct input_mms, 1);
+	input_stream_init(&m->base, &input_plugin_mms, url);
+
 	m->mms = mmsx_connect(NULL, NULL, url, 128 * 1024);
 	if (m->mms == NULL) {
 		g_free(m);
-		g_warning("mmsx_connect() failed");
-		return false;
+		g_set_error(error_r, mms_quark(), 0, "mmsx_connect() failed");
+		return NULL;
 	}
 
 	m->eof = false;
 
 	/* XX is this correct?  at least this selects the ffmpeg
 	   decoder, which seems to work fine*/
-	is->mime = g_strdup("audio/x-ms-wma");
+	m->base.mime = g_strdup("audio/x-ms-wma");
 
-	is->plugin = &input_plugin_mms;
-	is->data = m;
-	is->ready = true;
-	return true;
+	m->base.ready = true;
+
+	return &m->base;
 }
 
 static size_t
-input_mms_read(struct input_stream *is, void *ptr, size_t size)
+input_mms_read(struct input_stream *is, void *ptr, size_t size,
+	       GError **error_r)
 {
-	struct input_mms *m = is->data;
+	struct input_mms *m = (struct input_mms *)is;
 	int ret;
 
 	ret = mmsx_read(NULL, m->mms, ptr, size);
 	if (ret <= 0) {
 		if (ret < 0) {
-			is->error = errno;
-			g_warning("mmsx_read() failed: %s", g_strerror(errno));
+			g_set_error(error_r, mms_quark(), errno,
+				    "mmsx_read() failed: %s",
+				    g_strerror(errno));
 		}
 
 		m->eof = true;
@@ -91,29 +103,25 @@ input_mms_read(struct input_stream *is, void *ptr, size_t size)
 static void
 input_mms_close(struct input_stream *is)
 {
-	struct input_mms *m = is->data;
+	struct input_mms *m = (struct input_mms *)is;
 
 	mmsx_close(m->mms);
+	input_stream_deinit(&m->base);
 	g_free(m);
 }
 
 static bool
 input_mms_eof(struct input_stream *is)
 {
-	struct input_mms *m = is->data;
+	struct input_mms *m = (struct input_mms *)is;
 
 	return m->eof;
 }
 
-static int
-input_mms_buffer(G_GNUC_UNUSED struct input_stream *is)
-{
-	return 0;
-}
-
 static bool
 input_mms_seek(G_GNUC_UNUSED struct input_stream *is,
-	       G_GNUC_UNUSED off_t offset, G_GNUC_UNUSED int whence)
+	       G_GNUC_UNUSED goffset offset, G_GNUC_UNUSED int whence,
+	       G_GNUC_UNUSED GError **error_r)
 {
 	return false;
 }
@@ -122,7 +130,6 @@ const struct input_plugin input_plugin_mms = {
 	.name = "mms",
 	.open = input_mms_open,
 	.close = input_mms_close,
-	.buffer = input_mms_buffer,
 	.read = input_mms_read,
 	.eof = input_mms_eof,
 	.seek = input_mms_seek,
