@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * Copyright (C) 2003-2010 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,18 +31,20 @@
  * Codes for the type of a tag item.
  */
 enum tag_type {
-	TAG_ITEM_ARTIST,
-	TAG_ITEM_ALBUM,
-	TAG_ITEM_ALBUM_ARTIST,
-	TAG_ITEM_TITLE,
-	TAG_ITEM_TRACK,
-	TAG_ITEM_NAME,
-	TAG_ITEM_GENRE,
-	TAG_ITEM_DATE,
-	TAG_ITEM_COMPOSER,
-	TAG_ITEM_PERFORMER,
-	TAG_ITEM_COMMENT,
-	TAG_ITEM_DISC,
+	TAG_ARTIST,
+	TAG_ARTIST_SORT,
+	TAG_ALBUM,
+	TAG_ALBUM_ARTIST,
+	TAG_ALBUM_ARTIST_SORT,
+	TAG_TITLE,
+	TAG_TRACK,
+	TAG_NAME,
+	TAG_GENRE,
+	TAG_DATE,
+	TAG_COMPOSER,
+	TAG_PERFORMER,
+	TAG_COMMENT,
+	TAG_DISC,
 
 	TAG_MUSICBRAINZ_ARTISTID,
 	TAG_MUSICBRAINZ_ALBUMID,
