@@ -300,6 +300,9 @@ stat_directory(const struct directory *directory, struct stat *st)
 	if (path_fs == NULL)
 		return -1;
 	ret = stat(path_fs, st);
+	if (ret < 0)
+		g_warning("Failed to stat %s: %s", path_fs, g_strerror(errno));
+
 	g_free(path_fs);
 	return ret;
 }
@@ -316,6 +319,9 @@ stat_directory_child(const struct directory *parent, const char *name,
 		return -1;
 
 	ret = stat(path_fs, st);
+	if (ret < 0)
+		g_warning("Failed to stat %s: %s", path_fs, g_strerror(errno));
+
 	g_free(path_fs);
 	return ret;
 }
@@ -557,6 +563,7 @@ directory_child_access(const struct directory *directory,
 	/* access() is useless on WIN32 */
 	(void)directory;
 	(void)name;
+	(void)mode;
 	return true;
 #else
 	char *path = map_directory_child_fs(directory, name);
